"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const input_ui_ipc_provider_1 = require("input-ui-ipc-provider");
const ts_simple_ast_1 = require("ts-simple-ast");
const typescript_ast_util_1 = require("typescript-ast-util");
const typescript_plugins_text_based_user_interaction_1 = require("typescript-plugins-text-based-user-interaction");
const util_1 = require("./util");
const input_ui_ipc_provider_2 = require("input-ui-ipc-provider");
class SignatureAbstractCodeFix {
    constructor(options) {
        this.options = options;
        input_ui_ipc_provider_2.setLogger(options.log);
        this.inputConsumer = input_ui_ipc_provider_2.getInputConsumer();
        this.interactionTool = typescript_plugins_text_based_user_interaction_1.create(this.getTextUIToolConfig());
        options.log('SignatureAbstractCodeFix constructor finish');
    }
    predicate(arg) {
        this.inputConsumer.askSupported();
        this.targetInfo = this.getTargetInfo(arg.sourceFile, typescript_ast_util_1.positionOrRangeToNumber(arg.positionOrRange));
        if (!this.targetInfo) {
            arg.log('predicate false because getTargetNameAndReorder did not found anything');
            return false;
        }
        return true;
    }
    description(arg) {
        return this.name;
    }
    getCompletionsAtPosition(fileName, position, options) {
        return this.interactionTool.getCompletionsAtPosition(fileName, position, options);
    }
    getTargetInfo(sourceFile, position) {
        return util_1.getTargetInfo(sourceFile, position, (targetNode) => !!targetNode);
    }
    getSimpleTargetNode(file, position, name, log) {
        let expr = file.getDescendantAtPos(position);
        if (!expr) {
            return;
        }
        const predicate = (e) => (ts_simple_ast_1.TypeGuards.isSignaturedDeclaration(e) || ts_simple_ast_1.TypeGuards.isPropertySignature(e) || ts_simple_ast_1.TypeGuards.isFunctionLikeDeclaration(e)) && (ts_simple_ast_1.TypeGuards.isNamedNode(e) || ts_simple_ast_1.TypeGuards.isNameableNode(e) || ts_simple_ast_1.TypeGuards.isPropertyNamedNode(e)) && (!name || e.getName() === name);
        const e = [expr].concat(expr.getAncestors()).find(predicate);
        if (!e) {
            return;
        }
        return e;
    }
    getApplicableRefactors(info, refactorName, refactorActionName, fileName, positionOrRange, userPreferences) {
        this.inputConsumer.askSupported();
        const applicableRefactors = this.interactionTool.getApplicableRefactors(info, refactorName, refactorActionName, fileName, positionOrRange, userPreferences);
        this.selectedAction = applicableRefactors.selectedAction;
        // if there is no textUITool special comment it could be still a third party input provider ?
        if (!applicableRefactors.selectedAction && this.targetInfo && this.inputConsumer.hasSupport(input_ui_ipc_provider_1.INPUT_ACTIONS.inputText)) {
            applicableRefactors.refactors.push({
                name: refactorName,
                description: `${refactorName} description`,
                actions: [{ name: refactorActionName, description: this.printRefactorSuggestionMessage(this.targetInfo) }]
            });
        }
        return applicableRefactors;
    }
    applyImpl(arg, fn) {
        const sourceFile = arg.simpleNode.getSourceFile();
        const funcDecl = this.getSimpleTargetNode(sourceFile, typescript_ast_util_1.positionOrRangeToNumber(arg.positionOrRange), this.targetInfo.name, this.options.log);
        if (!funcDecl) {
            this.options.log(`${this.name} applyImpl aborted because function ${this.targetInfo.name} cannot be found at ${arg.positionOrRange}`);
            return;
        }
        fn(funcDecl);
        sourceFile.saveSync();
    }
    /** heads up ! this is very generic - you must override it as reorderParams does */
    textUIToolConfigFactory(config) {
        const defaultConfig = {
            name: 'generic',
            args: ['name'],
            commentType: 'block',
            print: action => this.printRefactorSuggestionMessage(Object.assign({}, this.targetInfo, { name: action && action.args && action.args.name || this.targetInfo && this.targetInfo.name || this.name })),
            snippet: (fileName, position) => {
                const sourceFile = this.options.info.languageService.getProgram().getSourceFile(fileName);
                this.targetInfo = this.getTargetInfo(sourceFile, position);
                if (!this.targetInfo) {
                    return;
                }
                const help = this.config.helpComment ? this.helpComment() : '';
                return `${this.name}("${this.targetInfo.name}")${help}`;
            },
            nameExtra: (fileName, position) => {
                if (!this.targetInfo) {
                    return '';
                }
                return `of ${this.targetInfo.name}`;
            }
        };
        return {
            prefix: '&%&%',
            log: this.options.log,
            actions: [Object.assign({}, defaultConfig, config)]
        };
    }
}
exports.SignatureAbstractCodeFix = SignatureAbstractCodeFix;
//# sourceMappingURL=SignatureAbstractCodeFix.js.map