"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const input_ui_ipc_provider_1 = require("input-ui-ipc-provider");
const ts_simple_ast_1 = require("ts-simple-ast");
const refactors_1 = require("../refactors");
const SignatureAbstractCodeFix_1 = require("../SignatureAbstractCodeFix");
const util_1 = require("../util");
/**
 * # Description
 *
 *  Allows users to remove parameters from signature/function-like declarations, calls, etc. see README
 *
 * # TODO
 *
 *  * refactor a implementation method wont change its interface signature - super
 *  * constructors not supported
 *  * perhaps is better to offer a GUI multiple select ? !!!
 */
class removeParamsCodeFixImpl extends SignatureAbstractCodeFix_1.SignatureAbstractCodeFix {
    constructor() {
        super(...arguments);
        this.name = refactors_1.PLUGIN_NAME + '-removeParam';
        this.config = {
            help: false
        };
    }
    printRefactorSuggestionMessage(targetInfo) {
        return `Remove parameters of "${targetInfo ? targetInfo.name : this.targetInfo.name}"`;
    }
    getTextUIToolConfig() {
        return this.textUIToolConfigFactory({
            name: 'removeParams',
            args: ['name', 'remove'],
            snippet: (fileName, position) => {
                const sourceFile = this.options.info.languageService.getProgram().getSourceFile(fileName);
                this.targetInfo = util_1.getTargetInfo(sourceFile, position);
                if (!this.targetInfo) {
                    return;
                }
                this.remove = [0];
                const help = this.config.helpComment ? this.helpComment() : '';
                return `removeParams("${this.targetInfo.name}", [${this.remove.join(', ')}])${help}`;
            },
        });
    }
    getTargetInfo(sourceFile, position) {
        const targetInfoPredicate = (targetNode) => !(!targetNode || targetNode.parameters && !targetNode.parameters.length) &&
            !(!targetNode || targetNode.arguments && !targetNode.arguments.length);
        return util_1.getTargetInfo(sourceFile, position, targetInfoPredicate);
    }
    apply(arg) {
        if (!this.selectedAction && this.inputConsumer.hasSupport(input_ui_ipc_provider_1.INPUT_ACTIONS.inputText)) {
            this.inputConsumer.inputText({ prompt: 'Enter removeParam definition', placeHolder: '[1]' })
                .then(response => {
                const remove = JSON.parse(response.answer);
                this.applyImpl(arg, (n) => { removeParameters(n, remove, this.options.log); });
            }).catch(ex => {
                this.options.log('this.inputConsumer.inputText catch ' + ex);
            });
        }
        else {
            this.applyImpl(arg, (n) => { removeParameters(n, this.selectedAction.args.remove, this.options.log); });
        }
    }
    helpComment() {
        return `/* Help: [1] means remove second argument/parameter, [0, 2] means remove first and third, etc */`;
    }
}
exports.removeParamsCodeFixImpl = removeParamsCodeFixImpl;
function removeParameters(node, remove, log) {
    log(`reorderParameters called with reorder: [${remove.join(', ')}]`);
    util_1.applyToSignature(node, (argsOrParams, parent, returnValue) => removeArgsOrParams(remove, argsOrParams, parent, log), log);
}
function removeOne(index, parent) {
    //TODO: check index - user errors
    if (ts_simple_ast_1.TypeGuards.isCallExpression(parent)) {
        parent.removeArgument(index);
    }
    else {
        parent.getParameters()[index].remove();
    }
}
function removeArgsOrParams(remove, argsOrParams, parent, log) {
    for (let index = 0; index < remove.length; index++) {
        removeOne(remove[index], parent);
    }
}
//# sourceMappingURL=removeParam.js.map